@@ -3,12 +3,24 @@
 # Copyright 2011 Piston Cloud Computing, Inc.
 
 # All Rights Reserved.
+#
+#    Licensed under the Apache License, Version 2.0 (the "License"); you may
+#    not use this file except in compliance with the License. You may obtain
+#    a copy of the License at
+#
+#         http://www.apache.org/licenses/LICENSE-2.0
+#
+#    Unless required by applicable law or agreed to in writing, software
+#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
+#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
+#    License for the specific language governing permissions and limitations
+#    under the License.
+
 """
 OpenStack Client interface. Handles the REST calls and responses.
 """
 
 import logging
-import os
 import time
 
 import requests
@@ -19,9 +31,10 @@ except ImportError:
     import simplejson as json
 
 from novaclient import exceptions
+from novaclient.openstack.common.gettextutils import _
+from novaclient.openstack.common.py3kcompat import urlutils
 from novaclient import service_catalog
 from novaclient import utils
-from novaclient.openstack.common.py3kcompat import urlutils
 
 
 class HTTPClient(object):
@@ -36,13 +49,19 @@ class HTTPClient(object):
                  timings=False, bypass_url=None,
                  os_cache=False, no_cache=True,
                  http_log_debug=False, auth_system='keystone',
-                 auth_plugin=None,
+                 auth_plugin=None, auth_token=None,
                  cacert=None, tenant_id=None):
         self.user = user
         self.password = password
         self.projectid = projectid
         self.tenant_id = tenant_id
 
+        # This will be called by #_get_password if self.password is None.
+        # EG if a password can only be obtained by prompting the user, but a
+        # token is available, you don't want to prompt until the token has
+        # been proven invalid
+        self.password_func = None
+
         if auth_system and auth_system != 'keystone' and not auth_plugin:
             raise exceptions.AuthSystemNotFound(auth_system)
 
@@ -68,8 +87,8 @@ class HTTPClient(object):
 
         self.times = []  # [("item", starttime, endtime), ...]
 
-        self.management_url = None
-        self.auth_token = None
+        self.management_url = self.bypass_url or None
+        self.auth_token = auth_token
         self.proxy_token = proxy_token
         self.proxy_tenant_id = proxy_tenant_id
         self.keyring_saver = None
@@ -97,7 +116,7 @@ class HTTPClient(object):
                 rql.addHandler(ch)
                 # Since we have already setup the root logger on debug, we
                 # have to set it up here on WARNING (its original level)
-                # otherwise we will get all the requests logging messanges
+                # otherwise we will get all the requests logging messages
                 rql.setLevel(logging.WARNING)
         # requests within the same session can reuse TCP connections from pool
         self.http = requests.Session()
@@ -119,16 +138,17 @@ class HTTPClient(object):
     def reset_timings(self):
         self.times = []
 
-    def http_log_req(self, args, kwargs):
+    def http_log_req(self, method, url, kwargs):
         if not self.http_log_debug:
             return
 
         string_parts = ['curl -i']
-        for element in args:
-            if element in ('GET', 'POST', 'DELETE', 'PUT'):
-                string_parts.append(' -X %s' % element)
-            else:
-                string_parts.append(' %s' % element)
+
+        if not kwargs.get('verify', True):
+            string_parts.append(' --insecure')
+
+        string_parts.append(" '%s'" % url)
+        string_parts.append(' -X %s' % method)
 
         for element in kwargs['headers']:
             header = ' -H "%s: %s"' % (element, kwargs['headers'][element])
@@ -141,11 +161,10 @@ class HTTPClient(object):
     def http_log_resp(self, resp):
         if not self.http_log_debug:
             return
-        self._logger.debug(
-            "RESP: [%s] %s\nRESP BODY: %s\n",
-            resp.status_code,
-            resp.headers,
-            resp.text)
+        self._logger.debug(_("RESP: [%(status)s] %(headers)s\nRESP BODY: "
+                             "%(text)s\n"), {'status': resp.status_code,
+                                             'headers': resp.headers,
+                                             'text': resp.text})
 
     def request(self, url, method, **kwargs):
         kwargs.setdefault('headers', kwargs.get('headers', {}))
@@ -157,12 +176,12 @@ class HTTPClient(object):
             del kwargs['body']
         if self.timeout is not None:
             kwargs.setdefault('timeout', self.timeout)
+        kwargs['verify'] = self.verify_cert
 
-        self.http_log_req((url, method,), kwargs)
+        self.http_log_req(method, url, kwargs)
         resp = self.http.request(
             method,
             url,
-            verify=self.verify_cert,
             **kwargs)
         self.http_log_resp(resp)
 
@@ -180,7 +199,6 @@ class HTTPClient(object):
             try:
                 body = json.loads(resp.text)
             except ValueError:
-                pass
                 body = None
         else:
             body = None
@@ -214,9 +232,11 @@ class HTTPClient(object):
             return resp, body
         except exceptions.Unauthorized as e:
             try:
-                # frist discard auth token, to avoid the possibly expired
+                # first discard auth token, to avoid the possibly expired
                 # token being re-used in the re-authentication attempt
                 self.unauthenticate()
+                # overwrite bad token
+                self.keyring_saved = False
                 self.authenticate()
                 kwargs['headers']['X-Auth-Token'] = self.auth_token
                 resp, body = self._time_request(self.management_url + url,
@@ -225,6 +245,11 @@ class HTTPClient(object):
             except exceptions.Unauthorized:
                 raise e
 
+    def _get_password(self):
+        if not self.password and self.password_func:
+            self.password = self.password_func()
+        return self.password
+
     def get(self, url, **kwargs):
         return self._cs_request(url, 'GET', **kwargs)
 
@@ -240,7 +265,8 @@ class HTTPClient(object):
     def _extract_service_catalog(self, url, resp, body, extract_token=True):
         """See what the auth service told us and process the response.
         We may get redirected to another site, fail or actually get
-        back a service catalog with a token and our endpoints."""
+        back a service catalog with a token and our endpoints.
+        """
 
         # content must always present
         if resp.status_code == 200 or resp.status_code == 201:
@@ -262,13 +288,14 @@ class HTTPClient(object):
                 self.management_url = management_url.rstrip('/')
                 return None
             except exceptions.AmbiguousEndpoints:
-                print("Found more than one valid endpoint. Use a more "
-                      "restrictive filter")
+                print(_("Found more than one valid endpoint. Use a more "
+                      "restrictive filter"))
                 raise
             except KeyError:
                 raise exceptions.AuthorizationFailure()
             except exceptions.EndpointNotFound:
-                print("Could not find any suitable endpoint. Correct region?")
+                print(_("Could not find any suitable endpoint. Correct "
+                        "region?"))
                 raise
 
         elif resp.status_code == 305:
@@ -291,7 +318,7 @@ class HTTPClient(object):
         # GET ...:5001/v2.0/tokens/#####/endpoints
         url = '/'.join([url, 'tokens', '%s?belongsTo=%s'
                         % (self.proxy_token, self.proxy_tenant_id)])
-        self._logger.debug("Using Endpoint URL: %s" % url)
+        self._logger.debug(_("Using Endpoint URL: %s") % url)
         resp, body = self._time_request(
             url, "GET", headers={'X-Auth-Token': self.auth_token})
         return self._extract_service_catalog(url, resp, body,
@@ -309,19 +336,16 @@ class HTTPClient(object):
                 self.version = part
                 break
 
+        if self.auth_token and self.management_url:
+            self._save_keys()
+            return
+
         # TODO(sandy): Assume admin endpoint is 35357 for now.
         # Ideally this is going to have to be provided by the service catalog.
         new_netloc = netloc.replace(':%d' % port, ':%d' % (35357,))
         admin_url = urlutils.urlunsplit(
             (scheme, new_netloc, path, query, frag))
 
-        # FIXME(chmouel): This is to handle backward compatibiliy when
-        # we didn't have a plugin mechanism for the auth_system. This
-        # should be removed in the future and have people move to
-        # OS_AUTH_SYSTEM=rackspace instead.
-        if "NOVA_RAX_AUTH" in os.environ:
-            self.auth_system = "rackspace"
-
         auth_url = self.auth_url
         if self.version == "v2.0":  # FIXME(chris): This should be better.
             while auth_url:
@@ -359,8 +383,13 @@ class HTTPClient(object):
         elif not self.management_url:
             raise exceptions.Unauthorized('Nova Client')
 
+        self._save_keys()
+
+    def _save_keys(self):
         # Store the token/mgmt url in the keyring for later requests.
-        if self.keyring_saver and self.os_cache and not self.keyring_saved:
+        if (self.keyring_saver and self.os_cache and not self.keyring_saved
+                and self.auth_token and self.management_url
+                and self.tenant_id):
             self.keyring_saver.save(self.auth_token,
                                     self.management_url,
                                     self.tenant_id)
@@ -372,7 +401,7 @@ class HTTPClient(object):
             raise exceptions.NoTokenLookupException()
 
         headers = {'X-Auth-User': self.user,
-                   'X-Auth-Key': self.password}
+                   'X-Auth-Key': self._get_password()}
         if self.projectid:
             headers['X-Auth-Project-Id'] = self.projectid
 
@@ -401,7 +430,7 @@ class HTTPClient(object):
         else:
             body = {"auth": {
                     "passwordCredentials": {"username": self.user,
-                                            "password": self.password}}}
+                                            "password": self._get_password()}}}
 
         if self.tenant_id:
             body['auth']['tenantId'] = self.tenant_id
@@ -412,17 +441,18 @@ class HTTPClient(object):
 
     def _authenticate(self, url, body, **kwargs):
         """Authenticate and extract the service catalog."""
+        method = "POST"
         token_url = url + "/tokens"
 
         # Make sure we follow redirects when trying to reach Keystone
-        resp, body = self._time_request(
+        resp, respbody = self._time_request(
             token_url,
-            "POST",
+            method,
             body=body,
             allow_redirects=True,
             **kwargs)
 
-        return self._extract_service_catalog(url, resp, body)
+        return self._extract_service_catalog(url, resp, respbody)
 
 
 def get_client_class(version):
@@ -434,8 +464,9 @@ def get_client_class(version):
     try:
         client_path = version_map[str(version)]
     except (KeyError, ValueError):
-        msg = "Invalid client version '%s'. must be one of: %s" % (
-              (version, ', '.join(version_map.keys())))
+        msg = _("Invalid client version '%(version)s'. must be one of: "
+                "%(keys)s") % {'version': version,
+                               'keys': ', '.join(version_map.keys())}
         raise exceptions.UnsupportedVersion(msg)
 
     return utils.import_class(client_path)
